<?php

/** -----------------------------------------------------------------
/// API-Endpoints
----------------------------------------------------------------- **/

class GetbackPushApiController
{
	/**
	 * @var string
	 */
	private $namespace;

	public function __construct()
    {
        $this->namespace = 'rest/v2/getback';
    }

    public function init()
    {
        add_action('rest_api_init', array($this, 'register_routes'));
    }


    public function register_routes()
    {
        // Connection Test Route
        register_rest_route($this->namespace, '/ping/', [
            'methods'  => 'POST',
            'callback' => array($this, 'test_connection'),
            'args' => [
                '_nonce' => [
                    'validate_callback' => function ($value) {
                        return wp_verify_nonce($value, 'wp_rest');
                    },
                    'required' => true,
                ]
            ],
            'permission_callback' => function () {
                return is_user_logged_in();
            }
        ]);

        // Send Web Push Route
        register_rest_route($this->namespace, '/message/', [
            'methods'  => 'POST',
            'callback' => array($this, 'send_notification'),
            'args' => [
                '_nonce' => [
                    'validate_callback' => function ($value) {
                        return (wp_verify_nonce($value, 'wp_rest')) ? true : new WP_Error('invalid', 'Invalid token');
                    },
                    'required' => true,
                ],
                'title' => [
                    'validate_callback' => function ($value) {
                        return is_string($value) && $value !== '' ? $value : new WP_Error('invalid', 'Title can not be empty');
                    },
                    'sanitize_callback' => function ($value) {
                        return sanitize_text_field($value);
                    },
                    'required' => true
                ],
                'body' => [
                    'validate_callback' => function ($value) {
                        return is_string($value) && $value !== '' ? $value : new WP_Error('invalid', 'Message body can not be empty');
                    },
                    'sanitize_callback' => function ($value) {
                        return sanitize_text_field($value);
                    },
                    'required' => true
                ],
                'postId' => [
                    'validate_callback' => function ($value) {
                        return is_string($value);
                    },
                    'required' => true
                ],
                'ttl' => [
                    'validate_callback' => function ($value) {
                        $d = DateTime::createFromFormat('d.m.Y', $value);
                        return ($d && $d->format('d.m.Y')  === $value || $value === '') ? true : new WP_Error('invalid', 'Date must be in a valid format (dd.mm.yyyy)');
                    },
                    'sanitize_callback' => function ($value) {
                        if (!$value) {
                            return;
                        }
                        $d = DateTime::createFromFormat('d.m.Y', $value);
                        return $d->format('Y-m-d');
                    }
                ],
                'ttlTime' => [
                    'validate_callback' => function ($value) {
                        return (preg_match("#([0-1]{1}[0-9]{1}|[2]{1}[0-3]{1}):[0-5]{1}[0-9]{1}#", $value) || $value === '') ? true : new WP_Error('invalid', 'Time must be in a valid format (HH:mm)');
                    },
                    'sanitize_callback' => function ($value) {
                        return sanitize_text_field($value);
                    }
                ],
                'link' => [
                    'validate_callback' => function ($value) {
                        return wp_http_validate_url($value) || $value === '' ? true : new WP_Error('invalid', 'URL must be in a valid format (i.e. http(s)://www.example.com)');
                    },
                    'sanitize_callback' => function ($value) {
                        return esc_url_raw($value);
                    }
                ]
            ],
            'permission_callback' => function () {
                return is_user_logged_in();
            }
        ]);

        if (Getback_Helper::woocommerce_is_active()) {
            if (null === WC()->cart && function_exists('wc_load_cart')) {
                wc_load_cart();
            }

            register_rest_route($this->namespace, '/cart/', [
                'methods' => 'GET',
                'callback' => array($this, 'get_cart_content'),
                'permission_callback' => '__return_true',
            ]);
        }
    }


    public function test_connection($request)
    {
        global $wp_version;
        $getback_options = get_option('getback_options');



        if (!$getback_options['apikey_getback']) {
            return new WP_REST_Response([
                'success' => false,
                'data' => [
                    'msg' => 'Please provide an API Key'
                ],
            ], 401);
        }

        $url = 'https://api.getback.ch/v1/push/messages';

        $args = array('headers' => array(
            'Authorization' => 'Basic ' . $getback_options['apikey_getback']
        ));

        $response = wp_remote_get($url, $args);

        $body = json_decode(wp_remote_retrieve_body($response), true);
        $response_code = wp_remote_retrieve_response_code($response);

        if ($response_code !== 200) {
            return new WP_REST_Response([
                'success' => false,
                'data' => [
                    'response' => $response_code,
                    'body'     => $body,
                    'wp_version' => $wp_version,
                    'php_version' => phpversion()
                ],
            ], $response_code);
        }

        if ($response_code == 200) {
            return new WP_REST_Response([
                'success' => true,
                'data' => [
                    'response' => $response_code,
                    'body'     => $body['messages'][0]
                ],
            ], 200);
        }
    }

    public function composite_product_filter($cart_item, $product_details)
    {
        if (Getback_Helper::extendos_mix_match_bundles_is_active()) {
            if ($cart_item['mmBundleParent']) {
                $product_details->parentKey = $cart_item['mmBundleParent'];
                return $product_details;
            }
        }

        if (Getback_Helper::woocommerce_composite_is_active()) {
            if ($cart_item['composite_parent']) {
                $product_details->parentKey = $cart_item['composite_parent'];
            }
        }

        return $product_details;
    }

    public function get_product_id($cart_item)
    {
        if (Getback_Helper::extendos_mix_match_bundles_is_active() && $cart_item['product_id'] == 0) {
            return $cart_item['variation_id'];
        } else {
            return $cart_item['product_id'];
        }
    }

    public function get_cart_content(\WP_REST_Request $request)
    {
        $cart_content = array();

        foreach (WC()->cart->get_cart() as $cart_item_key => $cart_item) {

            $product = wc_get_product($this->get_product_id($cart_item));

            $product_details = new stdClass();
            $id = '';
            $image = '';
            $product_details->key = $cart_item_key;
            $total_price = 0;

            if ($product && $product->is_type('variable')) {

                $variable_product = new WC_Product_Variable($cart_item['product_id']);
                $variations = $variable_product->get_available_variations();

                foreach ($variations as $variation) {
                    if ($variation['variation_id'] == $cart_item['variation_id']) {
                        $id = $cart_item['variation_id'];
                        $image = $variation['image']['thumb_src'];
                    }
                }

                $product_details = $this->composite_product_filter($cart_item, $product_details);
            } elseif ($product && $product->is_type('simple')) {

                $id = $product->get_id();
                $image = wp_get_attachment_image_url($product->get_image_id());
                $product_details = $this->composite_product_filter($cart_item, $product_details);
            } elseif ($product && ($product->is_type('wooextmm') || $product->is_type('composite'))) {

                $id = $product->get_id();
                $image = wp_get_attachment_image_url($product->get_image_id()) ? wp_get_attachment_image_url($product->get_image_id()) : '';
            } else {
                continue;
            }

          
            if ($cart_item['line_subtotal'] > 0){
              $total_price = $cart_item['line_subtotal'] +  $cart_item['line_subtotal_tax'];
              $total_price = round(($total_price + 0.000001) * 20) / 20;
            }

            $product_details->id = $id;
            $product_details->title = $product->get_name();
            $product_details->description = $product->get_description();
            $product_details->quantity = $cart_item['quantity'];
            $product_details->price = $cart_item['line_subtotal'];
            $product_details->tax = $cart_item['line_subtotal_tax'] ? $cart_item['line_subtotal_tax'] : 0;
            $product_details->totalPrice = $total_price;
            $product_details->url = $product->get_permalink();
            $product_details->image = $image;

            array_push($cart_content, $product_details);
        }

        return new WP_REST_Response([
            'success' => true,
            'data' => [
                'response' => 200,
                'body' => $cart_content
            ]
        ], 200);
    }

    public function send_notification($request)
    {
        $data = [];
        $getback_options = get_option('getback_options');

        $post_id = $this->get_request_value('postId', $request);
        $data['title'] = $this->get_request_value('title', $request);
        $data['body'] = $this->get_request_value('body', $request);
        $data['link'] = $this->get_request_value('link', $request);

        $ttl_date  = $this->get_request_value('ttl', $request);
        $ttl_time =  $this->get_request_value('ttlTime', $request);

        $action_buttons = $this->get_request_value('action_buttons', $request);

        if ($action_buttons !== "" && is_array($action_buttons)) {
            $data['action_buttons'] = $action_buttons;
        }

        $getback_default_ttl = $getback_options['default_ttl'] ?? '';

        if ($data['link'] == '') {
            $data['link'] = get_permalink($post_id);
        }

        if (get_the_post_thumbnail_url($post_id, 'full')) {
            $data['image'] = get_the_post_thumbnail_url($post_id, 'full');
        }

        if (!empty($getback_options['image_getback_icon'])) {
            $data['icon'] = $getback_options['image_getback_icon'];
        }
        if (!empty($getback_options['image_getback_badge'])) {
            $data['badge'] = $getback_options['image_getback_badge'];
        }


        $array = [
            "is_draft" => false,
            "autohide" => false,
            "rate_limit" => false,
            "urgency" => "normal",
            "is_silent" => false,
            "is_hidden_in_widget" => false,
            "lang" => [
                substr(get_locale(), 0, 2) => $data,
            ],
        ];

        if (!empty($ttl_date)) {
            if (!empty($ttl_time)) {
                $ttl_date_time = $ttl_date . ' ' . $ttl_time;
            } else {
                $ttl_date_time = $ttl_date . ' 23:59';
            }
            $array['ttl_date'] = date('Y-m-d H:i', strtotime($ttl_date_time));
        } elseif (!empty($getback_default_ttl)) {
            $array['ttl'] = (int)$getback_default_ttl;
        }

        if (!$getback_options['apikey_getback']) {
            return new WP_REST_Response([
                'success' => false,
                'data' => [
                    'msg' => 'Please provide an API Key'
                ],
            ], 401);
        }

        $url = 'https://api.getback.ch/v1/push/message';

        $args = array(
            'method' => 'POST',
            'headers' => array(
                'Authorization' => 'Basic ' . $getback_options['apikey_getback'],
                'Content-Type' => 'application/json'
            ),
            'body' => json_encode($array)
        );

        $response = wp_remote_post($url, $args);

        $body = json_decode(wp_remote_retrieve_body($response), true);
        $response_code = wp_remote_retrieve_response_code($response);

        if ($response_code !== 201) {
            return new WP_REST_Response([
                'success' => false,
                'data' => [
                    'response' => $response_code,
                    'body'     => $body
                ],
            ], $response_code);
        }

        if ($response_code == 201) {
            $time_now = current_time('mysql');
            update_post_meta($post_id, '_getback_last_sent', $time_now);
            return new WP_REST_Response([
                'success' => true,
                'data' => [
                    'response' => $response_code,
                    'body'     => $body
                ],
            ], 201);
        }
    }

    public function get_request_value($key, $request,  $default_value = '')
    {
        return isset($request[$key]) ? $request[$key] : $default_value;
    }
}

$GetbackPushApiController = new GetbackPushApiController;
$GetbackPushApiController->init();
