<?php

defined('ABSPATH') or die('Not allowed to access directly.');

class Getback_Admin
{

  /**
   * The ID of this plugin.
   *
   * @since    1.9.0
   * @access   private
   * @var      string    $plugin_name    The ID of this plugin.
   */
  private $plugin_name;

  /**
   * The version of this plugin.
   *
   * @since    1.9.0
   * @access   private
   * @var      string    $version    The current version of this plugin.
   */
  private $version;

  /**
   * The settings options
   *
   * @since    1.9.0
   * @access   private
   * @var      array    $version    The current values of the settings options
   */
  private static $settings_options;


  /**
   * Initialize the class and set its properties.
   *
   * @since    1.9.0
   * @param      string    $plugin_name       The name of this plugin.
   * @param      string    $version    The version of this plugin.
   */
  public function __construct($plugin_name, $version)
  {

    $this->plugin_name = $plugin_name;
    $this->version = $version;

    self::$settings_options = get_option('getback_options');
  }


  /**
   * Register the stylesheets for the admin area.
   *
   * @since    1.9.0
   */
  public function enqueue_styles()
  {
    wp_enqueue_style($this->plugin_name, plugin_dir_url(__FILE__) . 'assets/css/getback-admin.css', array(), $this->version, 'all');
  }


  /**
   * Register the JavaScript for the admin area.
   *
   * @since    1.9.0
   */
  public function enqueue_scripts()
  {
    wp_enqueue_media();

    wp_register_script($this->plugin_name, plugin_dir_url(__FILE__) . 'assets/js/getback-admin.js');

    wp_localize_script($this->plugin_name, 'getback_ajax_object', array(
      'ajax_url' => admin_url('admin-ajax.php'),
      'api_nonce' => wp_create_nonce('wp_rest'),
      'api_url'   => site_url('/wp-json/rest/v2/'),
      'permalink' => get_permalink()
    ));

    wp_enqueue_script($this->plugin_name, array('jquery'), $this->version, false);

    wp_enqueue_script($this->plugin_name . 'jquery-ui', plugin_dir_url(__FILE__) . 'assets/js/jquery-ui.min.js', array('jquery'), $this->version, false);
  }

  public function run_on_transition_post_public($new_status, $old_status, $post)
  {
    if (!($new_status !== 'publish' || empty($post) || $post->post_type === 'page')) {
      self::send_notification($new_status, $old_status, $post);
    }
  }

  public static function send_notification($new_status, $old_status, $post)
  {
    try {

      // Check if is REST Request and leave function if true
      if (defined('REST_REQUEST') && REST_REQUEST) {
        return;
      }

      // Check if POST Data is available
      $has_post_data = !empty($_POST);

      // Get Default TTL from Settings Page if defined
      $getback_default_ttl = self::$settings_options['default_ttl'] ?? '';

      // Return if POST Data nonce is not valid
      if (
        $has_post_data &&
        !wp_verify_nonce((isset($_POST['getback_inner_allow_metabox_nonce']) ?
          sanitize_text_field($_POST['getback_inner_allow_metabox_nonce']) :
          ''), 'getback_inner_allow_metabox')
      ) {
        return;
      }

      $getback_post_meta_box_present = (get_post_meta($post->ID, '_getback_meta_box_present', true) === '1');

      $getback_post_meta_send_notification_checkbox_checked = (get_post_meta($post->ID, '_getback_send_on_publish', true) === '1');

      // Check if Meta Box was present when saving the post
      $getback_meta_box_present = ($has_post_data &&
        isset($_POST['getback_meta_box_present']) &&
        $_POST['getback_meta_box_present'] === 'true') ? true : false;

      $getback_send_notification_checkbox_checked = ($has_post_data &&
        isset($_POST['getback_send_on_publish']) &&
        $_POST['getback_send_on_publish'] === 'true') ? true : false;

      $saved_from_wordpress_editor = $getback_meta_box_present || $getback_post_meta_box_present;

      if ($saved_from_wordpress_editor) {

        $send_notification = (($has_post_data && $getback_send_notification_checkbox_checked) || (!$has_post_data && $getback_post_meta_send_notification_checkbox_checked));

        if ($send_notification) {

          // reset the post meta data
          update_post_meta($post->ID, '_getback_meta_box_present', false);

          update_post_meta($post->ID, '_getback_send_on_publish', false);

          /**
           *  Remove checkbox data from $_POST to prevent saving the checkbox value after sending the message
           *  
           * This post was just created or updated from the Wordpress Editor. We're going to use the data from $_POST.
           */

          $message_data = array();

          $fields = array(
            "is_draft" => false,
            "autohide" => false,
            "rate_limit" => false,
            "urgency" => "normal",
            "is_silent" => false,
            "is_hidden_in_widget" => false
          );

          if ($has_post_data) {

            if (array_key_exists('getback_meta_box_present', $_POST)) {
              unset($_POST['getback_meta_box_present']);
            }
            if (array_key_exists('getback_send_on_publish', $_POST)) {
              unset($_POST['getback_send_on_publish']);
            }

            $message_data['title'] = isset($_POST['getback_post_title']) ? $_POST['getback_post_title'] : '';

            $message_data['body'] = isset($_POST['getback_post_desc']) ? $_POST['getback_post_desc'] : '';

            $message_data['link'] = (isset($_POST['getback_post_url']) && $_POST['getback_post_url'] !== '') ? $_POST['getback_post_url'] : get_permalink($post->ID);

            $ttl_date = isset($_POST['getback_post_ttl']) ? $_POST['getback_post_ttl'] : '';
            $ttl_time = isset($_POST['getback_post_ttltime']) ? $_POST['getback_post_ttltime'] : '';

            if ($_POST['getback_action_buttons_enabled'] === "1") {
              $message_data['action_buttons'] = $_POST['getback_action_buttons'];
            }

            if (!empty($ttl_date)) {
              if (!empty($ttl_time)) {
                $ttl_date_time = $ttl_date . ' ' . $ttl_time;
              } else {
                $ttl_date_time = $ttl_date . ' 23:59';
              }
              $fields['ttl_date'] = date('Y-m-d H:i', strtotime($ttl_date_time));
            } elseif (!empty($getback_default_ttl)) {
              $fields['ttl'] = (int) $getback_default_ttl;
            }
          }

          /**
           * This is a scheduled post. 
           */
          if (!$has_post_data) {
            $message_data['title'] = get_post_meta($post->ID, '_getback_post_title', true);

            $message_data['body'] = get_post_meta($post->ID, '_getback_post_desc', true);

            $message_data['link'] = get_post_meta($post->ID, '_getback_post_url', true) !== '' ? get_post_meta($post->ID, '_getback_post_url', true) : get_permalink($post->ID);

            $ttl_date = get_post_meta($post->ID, '_getback_post_ttl', true);
            $ttl_time = get_post_meta($post->ID, '_getback_post_ttltime', true);

            if (get_post_meta($post->ID, '_getback_action_buttons_enabled', true) === "1") {
              $message_data['action_buttons'] =  get_post_meta($post->ID, '_getback_action_buttons', true);
            }

            if (!empty($ttl_date)) {
              if (!empty($ttl_time)) {
                $ttl_date_time = $ttl_date . '' . $ttl_time;
              } else {
                $ttl_date_time = $ttl_date . '23:59';
              }
              $fields['ttl_date'] = date('Y-m-d H:i', strtotime($ttl_date_time));
            } elseif (!empty($getback_default_ttl)) {
              $fields['ttl'] = (int) $getback_default_ttl;
            }
          }

          $postThumbnail = get_the_post_thumbnail_url($post->ID, 'full');

          if ($postThumbnail && $postThumbnail !== '') {
            $message_data['image'] = $postThumbnail;
          }

          if (!empty(self::$settings_options['image_getback_icon'])) {
            $message_data['icon'] = self::$settings_options['image_getback_icon'];
          }
          if (!empty(self::$settings_options['image_getback_badge'])) {
            $message_data['badge'] = self::$settings_options['image_getback_badge'];
          }

          $fields['lang'] = [
            substr(get_locale(), 0, 2) => $message_data,
          ];

          $getback_api_url = 'https://api.getback.ch/v1/push/message';

          $args = array(
            'method' => 'POST',
            'headers' => array(
              'Authorization' => 'Basic ' . self::$settings_options['apikey_getback'],
              'Content-Type' => 'application/json'
            ),
            'body' => json_encode($fields)
          );

          $response = wp_remote_post($getback_api_url, $args);

          $response_body = wp_remote_retrieve_body($response);
          $response_code = wp_remote_retrieve_response_code($response);

          if (is_wp_error(($response) || $response_body === '')) {
            self::getback_transient('Problems occured while sending your notification.', 'error', 86400);
            return;
          }

          if ($response_code !== 201) {
            self::getback_transient('Problems occured while sending your notification. ' . $response_body, 'error', 86400);
            return;
          }

          $response_body = json_decode($response_body);

          $time_now = current_time('mysql');

          update_post_meta($post->ID, '_getback_last_sent', $time_now);

          if (Getback_Admin::get_show_notification_on_message_sent()) {
            if (!empty($response_body->message_id)) {
              $notice_text = 'Go to the Getback Dashboard to see more details about your message: <a href="https://backend.getback.ch/push/message/show/' . $response_body->message_id . '" target="_blank">Message: ' . $response_body->message_id . '</a>';

              self::getback_transient('Successfully sent a notification. ' . $notice_text, 'success', 86400);
            }
          }
        }
      }
    } catch (Exception $e) {
      new WP_Error('err', __('Getback: There was an problem sending a notification.'));
    }
  }

  public static function getback_transient($message, $type, $expiration)
  {
    if ($type === 'error') {
      $class = 'notice getback-error-notice error';
    }
    if ($type === 'success') {
      $class = 'notice getback-success-notice notice-success';
    }
    set_transient('getback_transient_' . $type, '<div class="' . $class . '">
        <p><strong>Getback Push:</strong><em> ' . $message . '</em></p>
        </div>', $expiration);
  }

  /**
   * Add options page
   */
  public function create_plugin_settings_page()
  {
    add_menu_page(
      'Getback Settings',
      'Getback Settings',
      'manage_options',
      'getback-settings',
      array($this, 'getback_settings_page'),
      "data:image/svg+xml;base64," . base64_encode('<svg id="Ebene_1" data-name="Ebene 1" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 20 20"><path fill="black" d="M17.07,5.86a4.43,4.43,0,0,0-2.61-.58,4.31,4.31,0,0,0-3,1.54V2.41H10.33V8.25l-.7,1-1.52,2.2c-.09.12-.18.24-.28.35a4,4,0,0,1-2.76,1.4A3.24,3.24,0,0,1,3.92,13a3.4,3.4,0,0,1-1.09-.74l-.16-.17a3.54,3.54,0,0,1-.61-1,3.47,3.47,0,0,1-.22-1.36A3.33,3.33,0,0,1,2.1,8.41,3.49,3.49,0,0,1,3.78,6.64a3.31,3.31,0,0,1,.77-.25h0a3.36,3.36,0,0,1,3,1,3.43,3.43,0,0,1,.7,1.06A3.7,3.7,0,0,1,8.4,9a2.2,2.2,0,0,1,.06.37c0,.07.06.3,0,.35l.86-1.23a.67.67,0,0,0,.06-.12.59.59,0,0,0,0-.14,4.56,4.56,0,0,0-2.8-2.82,4.52,4.52,0,0,0-3.93.51A4.57,4.57,0,0,0,.71,9a5.06,5.06,0,0,0,0,1.29A5.34,5.34,0,0,0,1,11.55a4.55,4.55,0,0,0,1,1.51,4.62,4.62,0,0,0,1.52,1,3.94,3.94,0,0,0,1.72.28h0a4.35,4.35,0,0,0,1.78-.42,5,5,0,0,0,.81-.51h0a4.69,4.69,0,0,0,.64-.54v.75a2.71,2.71,0,0,1-.24,1.14,3.22,3.22,0,0,1-.71,1,3.52,3.52,0,0,1-1,.67,3.28,3.28,0,0,1-1.2.23,3.37,3.37,0,0,1-1-.16,3,3,0,0,1-.94-.5,3.38,3.38,0,0,1-.75-.71,2.46,2.46,0,0,1-.42-.84H1a4.32,4.32,0,0,0,.55,1.22,4.19,4.19,0,0,0,.9,1,4.4,4.4,0,0,0,1.24.68,5.25,5.25,0,0,0,1.46.27,4.68,4.68,0,0,0,1.65-.23,4.76,4.76,0,0,0,1.46-.8,3.69,3.69,0,0,0,1-1.23,3.36,3.36,0,0,0,.34-1.52V11.39l.7-1,1-1.47L12,8a4.12,4.12,0,0,1,1.41-1.24,3.3,3.3,0,0,1,2.06-.38A3.52,3.52,0,0,1,18.1,8.89a4,4,0,0,1,.12.84h0v.09a3.54,3.54,0,0,1-1.41,2.79,3.58,3.58,0,0,1-2.24.7,3.13,3.13,0,0,1-1.86-.65,3.4,3.4,0,0,1-1.28-2.18c-.06.11-.62,1-.73,1.21a3.87,3.87,0,0,0,1.79,2.12,4.64,4.64,0,0,0,4.47,0l.3-.18a4.58,4.58,0,0,0,2.09-3.85,4.56,4.56,0,0,0-2.28-4Z"/></svg>')
    );
  }

  /**
   * Generate Settings Page with all Sections
   *
   * @since    1.9.0
   */
  public function getback_settings_page()
  {
    ?>
    <div class="wrap">
      <?php

      $this->getback_settings_page_tab_general();
      $this->getback_settings_page_tab_testing();
      $this->getback_settings_general_info();

      ?>

    </div>
  <?php
  }

  /**
   * Settings Page: General Settings Section
   *
   * @since    1.9.0
   */
  private function getback_settings_page_tab_general()
  {    
    settings_errors();
  ?>
    <h2></h2>
    <div class="gb__header">
      <img alt="Getback logo" src="<?php echo plugin_dir_url(__FILE__) . 'assets/img/getback-logo.svg'; ?>" />
      <h1 class="getback-title"><?php echo __("Getback Optimize & Web Push Notifications", 'getback'); ?></h1>
      <div class="getback-version"><?php echo 'v' . $this->version ?></div>
    </div>
    <div class="gb__panel first">
      <h1><?php echo __("Settings", 'getback'); ?></h1>

      <form method="post" id="getback_settings_form" action="options.php">
        <?php
        // This prints out all hidden setting fields
        settings_fields('getback_option_group');
        do_settings_sections('getback-settings');
        submit_button();
        ?>
      </form>

    </div>
  <?php
  }

  /**
   * Settings Page: General Info Section 
   *
   * @since    1.9.0
   */
  private function getback_settings_general_info()
  {
  ?>
    <div class="gb__panel">
      <img alt="Getback logo" class="gb__logo" src="<?php echo plugin_dir_url(__FILE__) . '/assets/img/getback-logo.svg'; ?>" />
      <p><?php echo __("Getback technology has been created to enable clients two specific value-add features: Onsite Conversion Optimisation and Web Push Notifications.", 'getback'); ?></p>

      <h2><?php echo __("Getback Optimize", 'getback'); ?></h2>
      <p><?php echo __("Getback Optimize is an onsite technology that helps you raise your conversion rate via specific measures tailored to your online shop. This feature allows you to read the behavior of your website visitors and provides them with the right cue, supporting service or suitable action at the right time.", 'getback'); ?></p>

      <h2><?php echo __("Getback Web Push Notifications", 'getback'); ?></h2>
      <p><?php echo __("Getback Web Push Notifications allows you to contact your visitors directly by sending them a browser message at any time.", 'getback'); ?></p>
      <p><?php echo __("If you’re not a Getback Customer yet and would like to take advantage of our services, please contact us at:", 'getback'); ?></p>
      <ul>
        <li><?php echo __("adfocus GmbH", 'getback'); ?></li>
        <li><?php echo __("E-mail <a href=\"mailto:info@getback.app\">info@getback.app</a>", 'getback'); ?></li>
        <li><?php echo __("Phone +41 44 508 34 17", 'getback'); ?></li>
        <li><a href="https://www.getback.app/en/web-push-notifications/">https://www.getback.app/en/web-push-notifications/</a>
        </li>
        <li>&nbsp;</li>
      </ul>
    </div>
  <?php
  }


  /**
   * Settings Page: Tab with Test Button
   *
   * @since    1.9.0
   */
  private function getback_settings_page_tab_testing()
  {
  ?>
    <div class="gb__panel">
      <h1><?php echo __("Connection Test", 'getback'); ?></h1>
      <p><?php echo __("Click on the <strong>Test Connection</strong>-Button to test the connection between your server and the Getback API. Please make sure, the API Key has the permission to <strong>
Retrieve messages</strong>.") ?></p>
      <button id="getback_test_connection" class="button button-primary gb-btn" action=""><?php _e('Test Connection', 'getback'); ?> <span class="gb-loader"></span></button>
      <div class="gb-test-result">
      </div>
    </div>
  <?php
  }


  /**
   * Register Settings page and settings fields
   *
   * @since    1.9.0
   */
  public function setup_settings_page()
  {
    register_setting(
      'getback_option_group', // Option group
      'getback_options', // Option name
      array($this, 'sanitize') // Sanitize
    );

    add_settings_section(
      'setting_section_getback_general', // ID
      '', // Title
      array($this, 'print_section_info'), // Callback
      'getback-settings' // Page
    );
    add_settings_field(
      'unique_getback_id',
      __('Getback ID', 'getback'),
      array($this, 'unique_getback_id_callback'),
      'getback-settings',
      'setting_section_getback_general'
    );
    add_settings_field(
      'apikey_getback',
      __('Getback API Key', 'getback'),
      [$this, 'apikey_callback'],
      'getback-settings',
      'setting_section_getback_general'
    );
    add_settings_field(
      'activate_getback_button',
      __('Activate Getback Script', 'getback'),
      [$this, 'toggler_onoff_callback'],
      'getback-settings',
      'setting_section_getback_general'
    );

    if (Getback_Helper::woocommerce_is_active()) {
      add_settings_field(
        'activate_getback_conversion_tracking_button',
        __('Activate Getback Conversion Tracking', 'getback'),
        [$this, 'toggler_conversion_onoff_callback'],
        'getback-settings',
        'setting_section_getback_general'
      );
    }

    add_settings_section(
      'setting_section_getback_images', // ID
      '', // Title
      array($this, 'print_section_images'), // Callback
      'getback-settings' // Page
    );
    add_settings_field(
      'image_getback_icon',
      __('Icon', 'getback'),
      [$this, 'upload_image_icon'],
      'getback-settings',
      'setting_section_getback_images'
    );
    add_settings_field(
      'image_getback_badge',
      __('Badge/app icon', 'getback'),
      [$this, 'upload_image_badge'],
      'getback-settings',
      'setting_section_getback_images'
    );

    add_settings_section(
      'setting_section_getback_additional_settings',
      '',
      array($this, 'print_section_additional_settings'),
      'getback-settings'
    );

    add_settings_field(
      'enabled_posttypes',
      __('Enabled Post Types', 'getback'),
      array($this, 'enabled_posttypes_callback'),
      'getback-settings',
      'setting_section_getback_additional_settings'
    );
    add_settings_field(
      'show_notification_on_message_sent',
      __('Show notification on message sent', 'getback'),
      [$this, 'show_notification_on_message_sent_callback'],
      'getback-settings',
      'setting_section_getback_additional_settings'
    );
    add_settings_field(
      'default_ttl',
      __('Default TTL', 'getback'),
      [$this, 'default_ttl_callback'],
      'getback-settings',
      'setting_section_getback_additional_settings'
    );
  }

  /**
   * Sanitize each setting field as needed
   *
   * @param array $input Contains all settings fields as array keys
   * @return array
   */
  public function sanitize($input)
  {
    $new_input = array();
    if (isset($input['unique_getback_id'])) {
      $new_input['unique_getback_id'] = sanitize_text_field($input['unique_getback_id']);
    }

    if (isset($input['onofftoggler'])) {
      $new_input['onofftoggler'] = (bool) $input['onofftoggler'];
    }

    if (isset($input['onofftoggler_conversion'])) {
      $new_input['onofftoggler_conversion'] = (bool) $input['onofftoggler_conversion'];
    }

    if (isset($input['image_getback_icon'])) {
      $new_input['image_getback_icon'] = sanitize_text_field($input['image_getback_icon']);
    }

    if (isset($input['image_getback_badge'])) {
      $new_input['image_getback_badge'] = sanitize_text_field($input['image_getback_badge']);
    }

    if (isset($input['apikey_getback'])) {
      // Only save API Key if changed. This prevents from saving the masked API Key value to the database.
      if (self::get_masked_api_key(self::$settings_options['apikey_getback']) != $input['apikey_getback']) {
        $new_input['apikey_getback'] = (string) $input['apikey_getback'];
      } else {
        $new_input['apikey_getback'] = self::$settings_options['apikey_getback'];
      }
    }

    if (isset($input['enabled_posttypes'])) {
      $post_types = $input['enabled_posttypes'];
      $new_input['enabled_posttypes'] = [];
      foreach ($post_types as $post_type) {
        array_push($new_input['enabled_posttypes'], sanitize_text_field($post_type));
      }
    }

    if (isset($input['show_notification_on_message_sent'])) {
      $new_input['show_notification_on_message_sent'] = (bool) $input['show_notification_on_message_sent'];
    }

    if (isset($input['default_ttl'])) {
      if ((int) ($input['default_ttl']) <= 0) {
        add_settings_error('getback-notices', 'default_ttl_error', 'Default TTL must be greater than zero.');
      } else {
        $new_input['default_ttl'] = sanitize_text_field($input['default_ttl']);
      }
    }

    return $new_input;
  }


  /**
   * Print the Section text
   */
  public function print_section_info()
  {
    print __('<p>Insert your Getback API Key below to get started. After that it is possible to activate Getback Web Push for your site to let the plugin handle the integration of the script tag into your site. <br>If you wish to integrate the script tag by yourself just leave the checkbox "Activate Getback Script" unchecked.</p>', 'getback');
  }

  /**
   * Print the Section Images text
   */
  public function print_section_images()
  {
    printf('<h2>' . __('Push Notification Icons:', 'getback') . '</h2>');
  }

  public function toggler_onoff_callback()
  {
    printf(
      '<input type="checkbox" id="onofftoggler" name="getback_options[onofftoggler]" %s />',
      $this->get_onofftoggler_setting() ? 'checked' : ''
    );
  }

  public function toggler_conversion_onoff_callback()
  {

    printf(
      '<input type="checkbox" id="onofftoggler_conversion" name="getback_options[onofftoggler_conversion]" %s />',
      $this->get_onofftoggler_conversion_setting() ? 'checked' : ''
    );
  }

  public function apikey_callback()
  {
    printf(
      '<input type="text" class="gb-form-control" id="apikey_getback" name="getback_options[apikey_getback]" value="%s" />',
      (isset(self::$settings_options['apikey_getback']) && self::$settings_options['apikey_getback'] !== '') ? esc_attr(self::get_masked_api_key(self::$settings_options['apikey_getback'])) : ''
    );
    echo '<p>' . __('Enter your API Key if you want to send messages directly from the wordpress backend.', 'getback') . '</p>';
  }

  /**
   * Get the settings option array and print one of its values
   */
  public function unique_getback_id_callback()
  {
    printf(
      '<input type="text" class="gb-form-control" id="unique_getback_id" name="getback_options[unique_getback_id]" value="%s" />',
      isset(self::$settings_options['unique_getback_id']) ? esc_attr(self::$settings_options['unique_getback_id']) : ''
    );

    echo '<p>' . __('Enter your Getback ID which is provided to you by your adfocus Account Manager.', 'getback') . '</p>';
  }

  public function upload_image_icon()
  {
    echo '<div class="getback-image-container">
                <div>';
    if (
      !empty(self::$settings_options['image_getback_icon'])
    ) {
      printf(
        '<img src="%s" style="width:100px;" alt="icon" id="image_getback_icon_placeholder">',
        esc_attr(self::$settings_options['image_getback_icon'])
      );
    }
    echo  '</div>
            <div>';
    printf(
      '<input type="file" id="image_getback_icon_form_input" name="image_getback_icon_form_input"/><label for="image_getback_icon_form_input" />Upload Icon</label>
                    <input type="hidden" id="image_getback_icon" name="getback_options[image_getback_icon]" value="%s" > ',
      esc_attr(self::$settings_options['image_getback_icon'] ?? '')
    );
    echo '<p>' . __('Square, scaled to 192x192px - max. File Size 10MB', 'getback') . '</p>';
    echo    '</div>
            </div>';
  }

  public function upload_image_badge()
  {
    echo '<div class="getback-image-container">
        <div>';
    if (!empty(self::$settings_options['image_getback_badge'])) {
      printf(
        '<img src="%s" style="width:100px;" alt="badge" id="image_getback_badge_placeholder">',
        esc_attr(self::$settings_options['image_getback_badge'])
      );
    }
    echo  '</div>
        <div>';
    printf(
      '<input type="file" id="image_getback_badge_form_input" name="image_getback_badge_form_input"/><label for="image_getback_badge_form_input" />Upload Badge</label>
                    <input type="hidden" id="image_getback_badge" name="getback_options[image_getback_badge]" value="%s" >',
      isset(self::$settings_options['image_getback_badge']) ? esc_attr(self::$settings_options['image_getback_badge']) : ''
    );
    echo '<p>' . __('Icon in notification bar on mobile, only Chrome on Android.', 'getback') . '</p>
        </div>
    </div>';
  }

  public function print_section_additional_settings()
  {
    printf('<h2>' . __('Additional Settings:', 'getback') . '</h2>');
  }

  public function enabled_posttypes_callback()
  {
    $post_types = get_post_types(['public' => true], 'names');
    echo __("Select the post types on which the Getback Integration should be visible:", 'getback')
  ?>

    <ul class="gb-posttypes-list">
      <?php
      foreach ($post_types as $post_type) {
      ?>
        <li>
          <input id="<?php echo $post_type ?>" <?php if (isset(self::$settings_options['enabled_posttypes']) && in_array($post_type, self::$settings_options['enabled_posttypes'])) echo 'checked=checked'; ?> type="checkbox" name="getback_options[enabled_posttypes][]" value="<?php echo $post_type ?>"><label for="<?php echo $post_type ?>"><?php echo $post_type ?></label>
        </li>
      <?php
      }
      ?>
    </ul>
    <?php
  }

  public function show_notification_on_message_sent_callback()
  {
    printf(
      '<input type="checkbox" id="onofftoggler" name="getback_options[show_notification_on_message_sent]" %s />',
      Getback_Admin::get_show_notification_on_message_sent() ? 'checked' : ''
    );
  }

  public function default_ttl_callback()
  {
    printf(
      '<div class="gb-input-group">
                <input class="gb-form-control" type="text" placeholder="4320" id="default_ttl" name="getback_options[default_ttl]" value="%s" />
                <div class="gb-input-group__append">
                    <span class="gb-input-group__text">minutes</span>
                </div>
            </div>',
      isset(self::$settings_options['default_ttl']) ? esc_attr(self::$settings_options['default_ttl']) : ''
    );

    echo '<p>' . __('Override Default TTL (default: 4320 / 3 days)', 'getback') . '</p>';
  }

  /**
   * Adding meta boxes to enabled post types
   *
   * @since    1.9.0
   */
  public function add_getback_meta_boxes_and_notifications($post_type)
  {
    function getback_admin_notice()
    {
      $error_transient = get_transient('getback_transient_error');
      $success_transient = get_transient('getback_transient_success');

      if (isset($error_transient)) {
        delete_transient('getback_transient_error');
        echo ($error_transient);
      }

      if (isset($success_transient)) {
        delete_transient('getback_transient_success');
        echo ($success_transient);
      }
    }
    add_action('admin_notices', 'getback_admin_notice');


    if (!isset(self::$settings_options['enabled_posttypes'])) {
      return;
    }

    $enabled_post_types = self::$settings_options['enabled_posttypes'];

    foreach ($enabled_post_types as $enabled_post_type) {
      add_meta_box(
        'getback_send_form_meta_box',
        __('Getback - Web Push Notifications', 'getback'),
        array($this, 'render_send_form_content'),
        $enabled_post_type,
        'advanced',
        'high'
      );

      add_meta_box(
        'getback_notify_on_post_meta_box',
        __('Getback - Web Push', 'getback'),
        array($this, 'render_notify_on_post_content'),
        $enabled_post_type,
        'side',
        'high'
      );
    }
  }

  /**
   * Render Meta Box content.
   *
   * @since 1.1.0
   * @param WP_Post $post The post object.
   */
  public function render_send_form_content($post)
  {

    // Add an nonce field so we can check for it later.
    wp_nonce_field('getback_inner_allow_metabox', 'getback_inner_allow_metabox_nonce');

    // Use get_post_meta to retrieve an existing value from the database.
    $getback_post_title = get_post_meta($post->ID, '_getback_post_title', true);
    $getback_post_desc = get_post_meta($post->ID, '_getback_post_desc', true);
    $getback_post_ttl = get_post_meta($post->ID, '_getback_post_ttl', true);
    $getback_post_ttltime = get_post_meta($post->ID, '_getback_post_ttltime', true);
    $getback_post_url = get_post_meta($post->ID, '_getback_post_url', true);
    $getback_action_buttons_enabled = intval(get_post_meta($post->ID, '_getback_action_buttons_enabled', true));
    $getback_action_buttons = get_post_meta($post->ID, '_getback_action_buttons', true);

    $getback_default_ttl = self::$settings_options['default_ttl'] ? self::$settings_options['default_ttl'] : 4320;
    $getback_default_ttl_string = self::calculateDayHoursMinutes($getback_default_ttl);

    $is_has_api_key = !empty(self::$settings_options['apikey_getback']);

    // Display the form, using the current value.
    if ($is_has_api_key) {
    ?>
      <div class="container-full-width">


        <div id="getback_result"></div><br />
        <div class="row gb__mb7">
          <label class="col-sm-12 col-md-1" for="getback_post_title"><?php _e('Title', 'getback'); ?></label>
          <input class="col-sm-12 col-md-8" type="text" name="getback_post_title" id="getback_post_title" value="<?php echo !empty($getback_post_title) ? $getback_post_title : ''; ?>">
        </div>
        <div class="row gb__mb7">
          <label class="col-sm-12 col-md-1" for="getback_post_desc"><?php _e('Message', 'getback'); ?></label>
          <textarea class="col-sm-11 col-md-8" name="getback_post_desc" id="getback_post_desc"><?php echo !empty($getback_post_desc) ? $getback_post_desc : ''; ?></textarea>
        </div>
        <div class="row gb__mb7">
          <label class="col-sm-12 col-md-1" for="getback_post_ttl"><?php _e('Expires on', 'getback'); ?></label>
          <input class="col-sm-6 col-md-2" type="text" name="getback_post_ttl" id="getback_post_ttl" placeholder="  dd.mm.yyyy" value="<?php echo !empty($getback_post_ttl) ? $getback_post_ttl : ''; ?>">
          <input class="col-sm-6 col-md-2" type="time" name="getback_post_ttltime" id="getback_post_ttltime" placeholder="hh:mm" value="<?php echo !empty($getback_post_ttltime) ? $getback_post_ttltime : ''; ?>">
          <span class="col-sm-12 col-md-4" id="getback_ttl_text"><?php _e('Default: ' . $getback_default_ttl . ' minutes / '  . $getback_default_ttl_string, 'getback'); ?></span>
        </div>
        <div class="row gb__mb7">
          <label class="col-sm-12 col-md-1" for="getback_post_url"><?php _e('Overwrite URL', 'getback'); ?></label>
          <input class="col-sm-11 col-md-8" type="url" name="getback_post_url" id="getback_post_url" placeholder="https://example.com" value="<?php echo !empty($getback_post_url) ? $getback_post_url : ''; ?>">
        </div>
        <div class="row gb__mb7">
          <label class="col-sm-12 col-md-1" for="getback_action_buttons_enabled"><?php _e('Action Buttons', 'getback') ?></label>
          <input type="checkbox" class="gb-btn-toggle gb-btn-toggle-js" name="getback_action_buttons_enabled" <?php echo $getback_action_buttons_enabled === 1 ? 'checked' : '' ?> id="getback_action_buttons_enabled" value="1">
        </div>
        <div class="gb-action-buttons <?php echo $getback_action_buttons_enabled === 1 ? 'visible' : '' ?>">
          <div class="row gb__mb7">
            <label class="col-sm-12 col-md-1" for="getback_action_button_1"><?php _e('Button 1', 'getback') ?></label>
            <input class="col-sm-12 col-md-3" placeholder="Button text" type="text" name="getback_action_buttons[0][text]" id="getback_post_action_button_text_1" value="<?php echo !empty($getback_action_buttons[0]['text']) ? $getback_action_buttons[0]['text'] : ''; ?>">
            <input class="col-sm-12 col-md-5" placeholder="Link" type="text" name="getback_action_buttons[0][link]" id="getback_post_action_button_url_1" value="<?php echo !empty($getback_action_buttons[0]['link'][0]) ? $getback_action_buttons[0]['link'] : ''; ?>">
          </div>
          <div class="row gb__mb7">
            <div class="col-md-5 col-md-offset-4 gb-action-buttons__subtext">
              <small><a onclick="gb.insertPermalink(1)" href="javascript:void(0);">Use Permalink</a></small>
            </div>
          </div>
          <div class="row gb__mb7">
            <label class="col-sm-12 col-md-1" for="getback_action_button_2"><?php _e('Button 2', 'getback') ?></label>
            <input class="col-sm-12 col-md-3" placeholder="Button text" type="text" name="getback_action_buttons[1][text]" id="getback_post_action_button_text_2" value="<?php echo !empty($getback_action_buttons[1]['text']) ? $getback_action_buttons[1]['text'] : ''; ?>">
            <input class="col-sm-12 col-md-5" placeholder="Link" type="text" name="getback_action_buttons[1][link]" id="getback_post_action_button_url_2" value="<?php echo !empty($getback_action_buttons[1]['link']) ? $getback_action_buttons[1]['link'] : ''; ?>">
          </div>
          <div class="row gb__mb7">
            <div class="col-md-5 col-md-offset-4 gb-action-buttons__subtext">
              <small><a onclick="gb.insertPermalink(2)" href="javascript:void(0);">Use Permalink</a></small>
            </div>
          </div>
        </div>

      </div>
      <br />
      <label class="gb__mb7 label-app" for="getback_send_button">
        &nbsp;
      </label>
      <button class="gb__mb7 components-button button button-primary gb-btn" name="getback_send_button" id="getback_send_button"><?php _e('Send push notification now', 'getback'); ?><span class="gb-loader"></span></button>
      <br />



    <?php
    } else {
    ?>
      <p>
        <a href="<?php echo admin_url('admin.php?page=getback-settings'); ?>"><?php echo __('Please add your API key here. ', 'getback'); ?></a>
      </p>
    <?php
    }
  }

  public static function calculateDayHoursMinutes($inputMinutes)
  {
    $inputMinutes = $inputMinutes * 60;
    $dtF = new \DateTime('@0');
    $dtT = new \DateTime("@$inputMinutes");
    return $dtF->diff($dtT)->format('%a days, %h hours, %i minutes');
  }

  public function render_notify_on_post_content($post)
  {
    $post_type = $post->post_type;
    $getback_send_on_publish = (bool) get_post_meta($post->ID, '_getback_send_on_publish', true);
    $last_sent_timestamp = get_post_meta($post->ID, '_getback_last_sent', true);

    if ($last_sent_timestamp) {
      $last_sent_timestamp = date_i18n(get_option('date_format') . ' ' . get_option('time_format') . ' (P)', strtotime($last_sent_timestamp));
    }

    ?>
    <input type="hidden" name="getback_meta_box_present" value="true"></input>
    <input type="checkbox" id="getback_send_on_publish" name="getback_send_on_publish" value="true" <?php echo $getback_send_on_publish ? 'checked' : '' ?>></input>
    <label>
      <?php if ($post->post_status === 'publish') {
        echo esc_attr('Send notification on ' . $post_type . ' update');
      } else {
        echo esc_attr('Send notification on ' . $post_type . ' publish');
      } ?>
    </label>
    <div class="gb-last-sent"><strong>Last sent:</strong> <span><?php echo $last_sent_timestamp ? $last_sent_timestamp : 'never'; ?></span></div>
<?php
  }

  public function show_admin_notices()
  {
    if (empty(self::$settings_options['unique_getback_id']) || empty(self::$settings_options['apikey_getback'])) {
      $class = 'notice notice-success';
      $url = get_admin_url() . '?page=getback-settings';
      printf('<div class="%1$s"><p><strong>Getback:</strong> Please add your API Key and Getback ID to start. Update <a href="%2$s">settings</a> now!</p></div>', esc_attr($class), esc_html($url));
    }
  }

  /**
   * Save the meta when the post is saved.
   *
   * @param int $post_id The ID of the post being saved.
   * @return int
   */
  public function on_save($post_id)
  {

    // Check if our nonce is set.
    if (!isset($_POST['getback_inner_allow_metabox_nonce'])) {
      return $post_id;
    }

    $nonce = $_POST['getback_inner_allow_metabox_nonce'];

    // Verify that the nonce is valid.
    if (!wp_verify_nonce($nonce, 'getback_inner_allow_metabox')) {
      return $post_id;
    }

    /*
         * If this is an autosave, our form has not been submitted,
         * so we don't want to do anything.
         */
    if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
      return $post_id;
    }

    // Check the user's permissions.
    if ('page' == $_POST['post_type']) {
      if (!current_user_can('edit_page', $post_id)) {
        return $post_id;
      }
    } else {
      if (!current_user_can('edit_post', $post_id)) {
        return $post_id;
      }
    }

    /* OK, it's safe for us to save the data now. */

    // Sanitize the user input.
    $getback_post_title = sanitize_text_field($_POST['getback_post_title']);
    $getback_post_desc = sanitize_text_field($_POST['getback_post_desc']);
    $getback_post_ttl = sanitize_text_field($_POST['getback_post_ttl']);
    $getback_post_ttltime = sanitize_text_field($_POST['getback_post_ttltime']);
    $getback_post_url = sanitize_text_field($_POST['getback_post_url']);
    $getback_action_buttons_enabled = $_POST['getback_action_buttons_enabled'] === "1" ? 1 : 0;
    $getback_action_buttons = $_POST['getback_action_buttons'];

    $getback_send_on_publish = isset($_POST['getback_send_on_publish']) ? filter_var($_POST['getback_send_on_publish'], FILTER_VALIDATE_BOOLEAN) : false;
    $getback_meta_box_present = isset($_POST['getback_meta_box_present']) ? filter_var($_POST['getback_meta_box_present'], FILTER_VALIDATE_BOOLEAN) : false;

    array_walk($getback_action_buttons, function ($value, $key) {
      $value['text'] = sanitize_text_field($value['text']);
      $value['link'] = sanitize_text_field($value['link']);
    });

    // Update the meta field.
    update_post_meta($post_id, '_getback_post_title', $getback_post_title);
    update_post_meta($post_id, '_getback_post_desc', $getback_post_desc);
    update_post_meta($post_id, '_getback_post_ttl', $getback_post_ttl);
    update_post_meta($post_id, '_getback_post_ttltime', $getback_post_ttltime);
    update_post_meta($post_id, '_getback_post_url', $getback_post_url);
    update_post_meta($post_id, '_getback_action_buttons_enabled', $getback_action_buttons_enabled);
    update_post_meta($post_id, '_getback_action_buttons', $getback_action_buttons);

    update_post_meta($post_id, '_getback_send_on_publish', $getback_send_on_publish);
    update_post_meta($post_id, '_getback_meta_box_present', $getback_meta_box_present);
  }

  public function get_onofftoggler_setting()
  {
    return (bool) isset(self::$settings_options['onofftoggler']) ? self::$settings_options['onofftoggler'] : false;
  }

  public function get_onofftoggler_conversion_setting()
  {
    return (bool) isset(self::$settings_options['onofftoggler_conversion']) ? self::$settings_options['onofftoggler_conversion'] : false;
  }

  public static function get_show_notification_on_message_sent()
  {
    return (bool) isset(self::$settings_options['show_notification_on_message_sent']) ? self::$settings_options['show_notification_on_message_sent'] : false;
  }

  public static function get_masked_api_key($getback_api_key)
  {
    return str_repeat('*', 21) . substr($getback_api_key, -7);
  }
}
