<?php

class Getback
{
	/**
	 * The ID of this plugin.
	 *
	 * @since    1.9.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

  /**
   * The loader that's responsible for maintaining and registering all hooks that power
   * the plugin.
   *
   * @since    1.9.0
   * @access   protected
   * @var      Getback_Loader    $loader    Maintains and registers all hooks for the plugin.
   */
  protected $loader;

  /**
   * The current version of the plugin.
   *
   * @since    1.9.0
   * @access   protected
   * @var      string    $version    The current version of the plugin.
   */
  protected $version;


  /**
   * Define the core functionality of the plugin.
   *
   * Set the plugin name and the plugin version that can be used throughout the plugin.
   * Load the dependencies, define the locale, and set the hooks for the admin area and
   * the public-facing side of the site.
   *
   * @since    1.9.0
   */
  public function __construct()
  {
    if (defined('GETBACK_VERSION')) {
      $this->version = GETBACK_VERSION;
    } else {
      $this->version = '1.0.0';
    }
    $this->plugin_name = 'getback';

    $this->load_dependencies();

    $this->define_admin_hooks();
    $this->define_public_hooks();
  }

  /**
   * Load the required dependencies for this plugin.
   *
   * Include the following files that make up the plugin:
   *
   * - Getback_Tracking_Loader. Orchestrates the hooks of the plugin.
   * - Getback_Tracking_Admin. Defines all hooks for the admin area.
   * - Getback_Tracking_Public. Defines all hooks for the public side of the site.
   *
   * Create an instance of the loader which will be used to register the hooks
   * with WordPress.
   *
   * @since    1.9.0
   * @access   private
   */
  private function load_dependencies()
  {
    /**
     * The class responsible for orchestrating the actions and filters of the
     * core plugin.
     */
    require_once plugin_dir_path(__FILE__) . 'class-getback-loader.php';

    /**
     * The class responsible for helper functions of the
     * core plugin.
     */
    require_once plugin_dir_path(__FILE__) . 'class-getback-helper.php';

    /**
     * The class responsible for defining all actions that occur in the admin area.
     */
    require_once plugin_dir_path(__FILE__) . 'class-getback-admin.php';

    /**
     * The class responsible for defining all actions that occur in the public-facing
     * side of the site.
     */
    require_once plugin_dir_path(__FILE__) . 'class-getback-public.php';

    /**
     * The class responsible for defining API Endpoints of the
     * core plugin
     */
    require_once plugin_dir_path(__FILE__) . 'api/routes.php';

    $this->loader = new Getback_Loader();
  }

  /**
   * Register all of the hooks related to the admin area functionality
   * of the plugin.
   *
   * @since    1.9.0
   * @access   private
   */
  private function define_admin_hooks()
  {
    // global $typenow;
    // $screen = get_current_screen();

    $plugin_admin = new Getback_Admin($this->get_plugin_name(), $this->get_version());

    $this->loader->add_action('admin_menu', $plugin_admin, 'create_plugin_settings_page');
    $this->loader->add_action('admin_menu', $plugin_admin, 'setup_settings_page');
    $this->loader->add_action('admin_init', $plugin_admin, 'add_getback_meta_boxes_and_notifications');
    $this->loader->add_action('save_post', $plugin_admin, 'on_save');
    $this->loader->add_action('transition_post_status', $plugin_admin, 'run_on_transition_post_public', 10, 3);


    $this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_styles');
    $this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts');
  }

  /**
   * Register all of the hooks related to the public-facing functionality
   * of the plugin.
   *
   * @since    1.9.0
   * @access   private
   */
  private function define_public_hooks()
  {

    $plugin_public = new Getback_Public($this->get_plugin_name(), $this->get_version());

    $this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_styles');
    $this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_scripts');

    $this->loader->add_action('wp_footer', $plugin_public, 'getback_toggler_script');
    $this->loader->add_action('woocommerce_thankyou', $plugin_public, 'getback_conversion_toggler_script');
  }

  /**
   * The name of the plugin used to uniquely identify it within the context of
   * WordPress and to define internationalization functionality.
   *
   * @since     1.9.0
   * @return    string    The name of the plugin.
   */
  public function get_plugin_name()
  {
    return $this->plugin_name;
  }

  /**
   * The reference to the class that orchestrates the hooks with the plugin.
   *
   * @since     1.9.0
   * @return    Getback_Loader    Orchestrates the hooks of the plugin.
   */
  public function get_loader()
  {
    return $this->loader;
  }

  /**
   * Retrieve the version number of the plugin.
   *
   * @since     1.0.0
   * @return    string    The version number of the plugin.
   */
  public function get_version()
  {
    return $this->version;
  }

  /**
   * Run the loader to execute all of the hooks with WordPress.
   *
   * @since    1.9.0
   */
  public function run()
  {
    $this->loader->run();
  }
}
